from datetime import datetime

from cpl_core.configuration import ConfigurationABC
from cpl_discord.events import OnReadyABC

from bot_core.logging.database_logger import DatabaseLogger
from bot_core.service.data_integrity_service import DataIntegrityService
from bot_data.service.seeder_service import SeederService


class DatabaseOnReadyEvent(OnReadyABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: DatabaseLogger,
        seeder: SeederService,
        data_integrity: DataIntegrityService,
    ):
        self._config = config

        self._logger = logger
        self._seeder = seeder
        self._data_integrity = data_integrity

        OnReadyABC.__init__(self)
        self._logger.info(__name__, f"Module {type(self)} loaded")

    def _validate_init_time(self):
        try:
            start_time = self._config.get_configuration("Database_StartTime")
            init_time = round(
                (datetime.now() - datetime.strptime(start_time, "%Y-%m-%d %H:%M:%S.%f")).total_seconds(),
                2,
            )
            self._config.add_configuration("Database_InitTime", str(init_time))
            self._logger.debug(__name__, f"Database Init time:  {init_time}s")
            # print warning if initialisation took too long
            if init_time >= 30:
                self._logger.warn(__name__, "It takes too long to prepare the database!")

            # print error if initialisation took way too long
            elif init_time >= 90:
                self._logger.error(__name__, "It takes far too long to prepare the database!!!")
        except Exception as e:  #
            self._logger.error(__name__, "Database init time calculation failed", e)
            return

    async def on_ready(self):
        self._logger.debug(__name__, f"Module {type(self)} started")

        await self._data_integrity.check_data_integrity()
        await self._seeder.seed()

        self._validate_init_time()
        self._logger.trace(__name__, f"Module {type(self)} stopped")
