import asyncio
import datetime

import bs4
import discord
import requests
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from cpl_translation import TranslatePipe
from discord.ext import tasks

from bot_core.abc.task_abc import TaskABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.logging.task_logger import TaskLogger
from bot_core.service.message_service import MessageService
from bot_data.abc.steam_special_offer_repository_abc import SteamSpecialOfferRepositoryABC
from bot_data.model.server_config import ServerConfig
from bot_data.model.steam_special_offer import SteamSpecialOffer


class SteamOfferWatcher(TaskABC):
    def __init__(
        self,
        config: ConfigurationABC,
        bot: DiscordBotServiceABC,
        logger: TaskLogger,
        db: DatabaseContextABC,
        offers: SteamSpecialOfferRepositoryABC,
        message_service: MessageService,
        t: TranslatePipe,
    ):
        TaskABC.__init__(self)

        self._config = config
        self._logger = logger
        self._db = db
        self._offers = offers
        self._bot = bot
        self._message_service = message_service
        self._t = t

        self._is_new = False
        self._urls = {}
        self._image_urls = {}

        self.watch.start()

    @staticmethod
    def _get_max_count() -> int:
        count = 0
        result = requests.get(f"https://store.steampowered.com/search/results?specials=1")
        soup = bs4.BeautifulSoup(result.text, "lxml")
        element = soup.find_all("div", {"class": "search_results_count"})
        if len(element) < 1:
            return count

        count = int(element[0].contents[0].split(" ")[0].replace(",", ""))

        return count

    def _get_games_from_page(self, start: int, count: int) -> List[SteamSpecialOffer]:
        games = List(SteamSpecialOffer)
        result = requests.get(
            f"https://store.steampowered.com/search/results?query&start={start}&count={count}&force_infinite=1&specials=1"
        )
        soup = bs4.BeautifulSoup(result.text, "lxml")
        elements = soup.find_all("a", {"class": "search_result_row"})
        if len(elements) < 1:
            return games

        for element in elements:
            name_element = element.find("span", {"class": "title"})
            original_price_element = element.find("div", {"class": "discount_original_price"})
            discount_element = element.find("div", {"class": "discount_pct"})
            discount_price_element = element.find("div", {"class": "discount_final_price"})

            if (
                name_element is None
                or len(name_element.contents) < 1
                or original_price_element is None
                or len(original_price_element.contents) < 1
                or discount_element is None
                or len(discount_element.contents) < 1
                or discount_price_element is None
                or len(discount_price_element.contents) < 1
            ):
                continue

            name = name_element.contents[0].replace("'", "`").replace('"', "`")
            original_price = float(
                original_price_element.contents[0].replace(" ", "").replace("€", "").replace(",", ".")
            )
            discount = int(discount_element.contents[0].replace("%", ""))
            discount_price = float(
                discount_price_element.contents[0].replace(" ", "").replace("€", "").replace(",", ".")
            )
            games.add(SteamSpecialOffer(name, original_price, discount_price, discount))
            self._urls[name] = element.attrs["href"]
            self._image_urls[name] = element.find("div", {"class": "search_capsule"}).find("img").attrs["src"]

        return games

    def _get_new_game_offers(self) -> List[SteamSpecialOffer]:
        new_offers = List(SteamSpecialOffer)

        # sale_count = self._get_max_count() + 100
        sale_count = 500  # only look at first 500
        self._logger.debug(__name__, f"Get special offers from 0 to {sale_count}")
        for i in range(0, sale_count, 100):
            new_offers.extend(self._get_games_from_page(i, 100))

        self._logger.debug(__name__, f"Got {new_offers.count()} offers")

        return new_offers

    async def _send_embed_for_offer(self, offer: SteamSpecialOffer, channel_id: int) -> discord.Embed:
        embed = discord.Embed(
            title=offer.name,
            url=self._urls[offer.name],
            color=int("ef9d0d", 16),
            timestamp=datetime.datetime.now(),
        )

        embed.add_field(
            name=self._t.transform("modules.special_offers.price"),
            value=f"~~{offer.original_price}€~~",
            inline=True,
        )
        embed.add_field(
            name=self._t.transform("modules.special_offers.discount"), value=f"{offer.discount_pct}%", inline=True
        )
        embed.add_field(
            name=self._t.transform("modules.special_offers.discount_price"),
            value=f"{offer.discount_price}€",
            inline=True,
        )

        embed.set_image(url=self._image_urls[offer.name])

        await self._message_service.send_channel_message(
            self._bot.get_channel(channel_id),
            embed,
            is_persistent=True,
        )

    def _watch(self) -> List[SteamSpecialOffer]:
        self._is_new = self._offers.get_steam_special_offers().count() == 0
        new_offers = self._get_new_game_offers()
        new_offers_names = new_offers.select(lambda x: x.name).to_list()

        old_offers = self._offers.get_steam_special_offers()
        old_offers_names = old_offers.select(lambda x: x.name).to_list()

        offers_for_notifications = List(SteamSpecialOffer)

        for offer in old_offers:
            offer: SteamSpecialOffer = offer
            if offer.name in new_offers_names:
                continue

            self._offers.delete_steam_special_offer(offer)
            self._db.save_changes()

        for offer in new_offers:
            if offer.name in old_offers_names:
                self._offers.update_steam_special_offer(offer)
                self._db.save_changes()
                continue

            self._offers.add_steam_special_offer(offer)
            self._db.save_changes()
            offers_for_notifications.add(offer)

        self._logger.trace(__name__, "Finished watching")
        return offers_for_notifications

    @tasks.loop(time=datetime.time(hour=16, minute=30))
    async def watch(self):
        self._logger.info(__name__, "Watching steam special offers")
        try:
            offers_for_notifications = self._watch()
            self._logger.debug(__name__, f"Sending offer notifications for {offers_for_notifications.count()} offers")
            if self._is_new:
                return
            for guild in self._bot.guilds:
                settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{guild.id}")
                if (
                    not FeatureFlagsSettings.get_flag_from_dict(
                        settings.feature_flags, FeatureFlagsEnum.steam_special_offers
                    )
                    or settings.game_offer_notification_chat_id is None
                ):
                    continue

                for offer in offers_for_notifications:
                    self._bot.loop.create_task(
                        self._send_embed_for_offer(offer, settings.game_offer_notification_chat_id)
                    )
        except Exception as e:
            self._logger.error(__name__, f"Steam offer watcher failed", e)

    @watch.before_loop
    async def wait(self):
        await self._wait_until_ready()
